/* ////////////////////////////////////////////////////////////////////////////
** File:      dsODO.c
*/                  
const unsigned char  Ver[] = "dsODO 1.1.0 by Guiott 03-08"; // 26+1 char
/* Author:    Guido Ottaviani-->g.ottaviani@mediaprogetti.it<--
** Description:navigation control through odometry with dsPIC30F3013   
** Descrizione:controllo movimento tramite odometria con dsPIC30F3013   
**
** Detailed descriptions are on file "descrEng.txt" 
** numbers between brackets, eg.: [1] , are the references to the specific 
** decription into the file
**
** I commenti dettagliati sono sul file "descr.txt" 
** il numero fra parentesi quadre, es.: [1] , e' il riferimento al relativo 
** commento nel documento di descrizione   
 
-------------------------------------------------------------------------------
Copyright 2008 Guido Ottaviani
g.ottaviani@mediaprogetti.it

	dsODO is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    dsODO is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with dsODO.  If not, see <http://www.gnu.org/licenses/>.
-------------------------------------------------------------------------------    
/////////////////////////////////////////////////////////////////////////////*/

#include "dsODO_definitions.h"

int main (void)
{
Settings();
ConstantsRead();	// get constant values from EEPROM

/*---------------------------------------------------------------------------*/
/* Variables initialisation        	    									 */
/*---------------------------------------------------------------------------*/
LED1 = 1;			// [1]
LED2 = 0;
CS1 = 0;
CS2 = 0;
TX_FLAG = 0;
TX_BUFF_FULL=0;		
TX2_FLAG = 0;
TX2_BUFF_FULL=0;		
UartRxStatus = 0;
UartRx2Status = 0;
ASK_PARAM_FLAG = 0;	// [20]
SPACE_FLAG=0;		// [21c]
ODO_FLAG=0;
ORIENTATION_FLAG=0;
AskParam = 1;
AskParamTimer1 = 0;
AskParamTimer2 = 0;
ChkSum=0;

ANGLE_PID_DES=0;
InitAnglePid();

DIST_PID_DES=0;
InitDistPid();
DIST_TIMER_FLAG = 0;
DIST_ENABLE_FLAG = 0;
VelDecr = 0;
ResetCount = 0;
EEPROM_WRITE_STATUS_FLAG = 0;
/*..................................................Variables initialisation */

ISR_Settings();	// Configures and enables ISRs

MC_Setting();	// Configures both motor controllers [16a]

//------- Enables Timer 1 interrupt	[13]
// Starts timing for Supervisor and Motor Controllers
ConfigIntTimer1(T1_INT_PRIOR_4 & T1_INT_ON);
PR1 = TMR1_VALUE;

// Echo(); // just for testing RF comm

/*===========================================================================*/
/* Main loop                                                                 */
/*===========================================================================*/
while (1)
{						
/* ----------------------------------------- one character coming from UART1 */	
if (UartRxPtrIn != UartRxPtrOut) UartRx();	// [6f]

/* ----------------------------------------- one character coming from UART2 */	
if (UartRx2PtrIn != UartRx2PtrOut) UartRx2();	// [6f]	

/* ------------ a command coming from serial interface 1 (Motor Controllers) */	 
if (UartRxStatus == 99) Parser();

/* --------------- a command coming from serial interface 2 (telemetry) [16c]*/	 
if (UartRx2Status == 99 && !AskParamStatus && (AskParamTimer1<ASK_PARAM_GUARD))
	Parser2();

/* ---------------- a string has to be sent on serial interface 2 towards MCs*/	 
if (TX_FLAG && U1STAbits.TRMT)	UartTx();

/* --------- a string has to be sent on serial interface 2 towards Main board*/	 
if (TX2_FLAG && U2STAbits.TRMT) UartTx2();

/* ------------------ it's time to ask parameters to Motor Controllers [20a] */	 
if (ASK_PARAM_FLAG && !TX_PENDING)	AskParamCall();

/* ----------------------------------------- DeadReckonig field mapping [21] */	 
if (ODO_FLAG) DeadReckoning();

/* ---------------------------------------------------- Orientation PID [23] */	 
if (ORIENTATION_FLAG) Orientation();

/* --------------------------------------------- Send VelDes[x] to MCs [23f] */	 
if (SendVelStatus && !TX_PENDING && U1STAbits.TRMT) SendVel();

/* ------------------------------------------------------- Distance PID [24] */	 
if (DIST_TIMER_FLAG && DIST_ENABLE_FLAG) Distance();


	if (Blink == 200)	// [1a]
	{
		LED1 = 0;
		Blink ++;
	}
	if (Blink == 1001)
	{
		LED1 = 1;
		Blink = 0;
	}
	
	IdleCount ++;	// [25]
	
}/*....Main loop*/

}/*.....................................................................Main */


/*===========================================================================*/
/* Functions                                                                 */
/*===========================================================================*/
void EepromWrite(char Row)//start writing of 16 words to EEPROM ROW #EErow [30]
{
	EErow = Row;
	_wait_eedata(); // just as a security if a previous saving is pending
	// first INT occurs and the rest of the write sequence is managed by ISR
	_erase_eedata(EE_addr+(ROW_LEN*2*EErow), _EE_ROW);
	EEPROM_WRITE_STATUS_FLAG = 1;	// next status write
}

void ConstantsRead(void)	// get constant values from EEPROM
{
// initialize EEPROM memory space
_init_prog_address(EE_addr, EEdata);	
   
//Copy data from EEPROM to RAM
_memcpy_p2d16(EEvalue, EE_addr, EEPROM_SIZE*2);   
  
/* Speed calculation K in micron/second = 298536736
	see dsPID folder: descrEng.txt [4] [7] [19]
	Speed calculation K in m/s as a power of 2 to semplify dsPID elaboration
	Kvel[] = (K << 15) / 1.000.000
	long, 2 x 4 bytes = 8 byte */
Kvel[0] = Eeprom2Long(0);					// EEvalue[0]-[1] EEPROM ROW # 0
Kvel[1] = Eeprom2Long(2);					// EEvalue[2]-[3] # 0
Kvel[0] = ((long long)Kvel[0] << 15)/1000000;
Kvel[1] = ((long long)Kvel[1] << 15)/1000000;

// KP, KI, KD for Angle PID	[23]
// fractional, 3 x 2 = 6 bytes
ANGLE_KP=Eeprom2Fract(4);					// EEvalue[4] # 0
ANGLE_KI=Eeprom2Fract(5);					// EEvalue[5] # 0
ANGLE_KD=Eeprom2Fract(6);					// EEvalue[6] # 0

// KP, KI, KD for Distance PID [24]
// fractional, 3 x 2 = 6 bytes
DIST_KP=Eeprom2Fract(7);					// EEvalue[7] # 0
DIST_KI=Eeprom2Fract(8);					// EEvalue[8] # 0
DIST_KD=Eeprom2Fract(9);					// EEvalue[9] # 0

// KP, KI, KD for Speed PID MC1 e MC2 (dsPID) in int x 10.000 
// int, 2MCs x 3params x 2bytes = 12 bytes
Kpid[0][0] = EEvalue[10];					// EEvalue[10] # 0
Kpid[0][1] = EEvalue[11];					// EEvalue[11] # 0
Kpid[1][0] = EEvalue[12];					// EEvalue[12] # 0
Kpid[1][1] = EEvalue[13];					// EEvalue[13] # 0
Kpid[2][0] = EEvalue[14];					// EEvalue[14] # 0
Kpid[2][1] = EEvalue[15];					// EEvalue[15] # 0

// constants for traveled distance calculation: SPACE_ENC_4X in mm [21]
// float, 2 x 4 = 8 bytes
Ksp[0] = Eeprom2Float(16);					// EEvalue[16]-[17] EEPROM ROW # 1
Ksp[1] = Eeprom2Float(18);					// EEvalue[18]-[19] # 1

// base width, distance between center of the wheels [21]
// float, 1 x 4 = 4 bytes
Axle = Eeprom2Float(20);					// EEvalue[20]-[21] # 1
#define SEMI_AXLE Axle/2

/* an alternative way to read EEprom data
TBLPAG = __builtin_tblpage(&EEdata);	// EEPROM memory space

// read data from EEPROM to buffer
for (i=0; i<(EEPROM_SIZE); i++)
{
eedata_read(EEdata,EEdata[i], EEvalue[i]);
}
*/
}

void Long2Eeprom (long Lvar, int EeIndx)	// [27]
{//put a 4 byte long variable in 2 EEPROM buff words starting from EeIndx pos.
	int *EEpoint;
	long *LongPoint;
	LongPoint = &Lvar;
	EEpoint=(int*)(LongPoint);
	EEvalue[EeIndx] = *(EEpoint);
	EEvalue[EeIndx+1] = *(EEpoint+1);
}

long Eeprom2Long (int EeIndx)	// [27]
{//get a 4 byte long variable from 2 EEPROM buf words starting from EeIndx pos. 
	int *EEpoint;
	long *LongPoint;
	long Lvar;
	LongPoint = &Lvar;
	EEpoint=(int*)(LongPoint);
	*(EEpoint) = EEvalue[EeIndx];
	*(EEpoint+1) = EEvalue[EeIndx+1];
	return Lvar;
}

void Float2Eeprom (float Fvar, int EeIndx)	// [27]
{//put a 4 byte float variable in 2 EEPROM buff words starting from EeIndx pos.
	int *EEpoint;
	float *FloatPoint;
	FloatPoint = &Fvar;
	EEpoint=(int*)(FloatPoint);
	EEvalue[EeIndx] = *(EEpoint);
	EEvalue[EeIndx+1] = *(EEpoint+1);
}

float Eeprom2Float (int EeIndx)	// [27]
{//get a 4 byte float variable from 2 EEPROM buf words starting from EeIndx pos 
	int *EEpoint;
	float *FloatPoint;
	float Fvar;
	FloatPoint = &Fvar;
	EEpoint=(int*)(FloatPoint);
	*(EEpoint) = EEvalue[EeIndx];
	*(EEpoint+1) = EEvalue[EeIndx+1];
	return Fvar;
}

void Fract2Eeprom (fractional Fvar, int EeIndx)	// [27]
{//put a 2 byte fract variable in 1 EEPROM buff words from EeIndx pos.
	int *EEpoint;
	fractional *FractPoint;
	FractPoint = &Fvar;
	EEpoint=(int*)(FractPoint);
	EEvalue[EeIndx] = *(EEpoint);
}

fractional Eeprom2Fract (int EeIndx)	// [27]
{// get a 2 byte fract variable from 1 EEPROM buff words at EeIndx pos. 
	int *EEpoint;
	fractional *FractPoint;
	fractional Fvar;
	FractPoint = &Fvar;
	EEpoint=(int*)(FractPoint);
	*(EEpoint) = EEvalue[EeIndx];
	return Fvar;
}
void I2cRX(void)
{
// ************************************** DA FAREEEEEEEEEEEEEEEEEEEEEE	
}

void Distance(void)	// [24]
{
	float DPosX;	// delta PosX
	float DPosY;	// delta PosY
	
	DIST_TIMER_FLAG = 0; // will be set again by timer ISR
	
	DPosX = PosXdes - PosXmes;	   // delta on X axis
	DPosY =	PosYdes - PosYmes;	   // delta on Y axis
	ThetaDes = atanf(DPosX/DPosY); // Tangent(ThetaDes) = Sin/Cos = X/Y
	Dist = sqrtf(powf(DPosX,2) + powf(DPosY,2));// Pythagora's Theorem
	if (Dist < MIN_DIST_PID)	   // [24c]
	{
		DIST_PID_DES = 0;		   // the goal is to have dist=0 from target
		DIST_PID_MES = Q15((float)Dist/(float)MIN_DIST_PID);  // measured input
		PID(&DistPIDstruct);
		VelDecr = VelDesM+(((long)(DIST_PID_OUT>>5)*VelDesM)>>10); // [24d]
	}
	else
	{
		VelDecr = 0;
	}
}

void InitDistPid(void)
{
//Initialize the PID data structure: PIDstruct
//Set up pointer to derived coefficients
DistPIDstruct.abcCoefficients = &DistabcCoefficient[0];
//Set up pointer to controller history samples
DistPIDstruct.controlHistory = &DistcontrolHistory[0]; 
// Clear the controler history and the controller output
PIDInit(&DistPIDstruct); 
//Derive the a,b, & c coefficients from the Kp, Ki & Kd
PIDCoeffCalc(&DistKCoeffs[0], &DistPIDstruct); 
}

void SendVel(void)	// // setting reference speed (as mm/s) [23f]
{
	if (SendVelStatus == 1)
	{
		CmdWriteVel(ID_R,VelDes[R]); // sends VelDes to Right MC
		SendVelStatus = 2;
	}
	else if (SendVelStatus == 2)
	{
		CmdWriteVel(ID_L,VelDes[L]); // sends VelDes to Left MC
		SendVelStatus = 0;
	}	
}

void InitAnglePid(void)
{
//Initialize the PID data structure: PIDstruct
//Set up pointer to derived coefficients
AnglePIDstruct.abcCoefficients = &AngleabcCoefficient[0];
//Set up pointer to controller history samples
AnglePIDstruct.controlHistory = &AnglecontrolHistory[0]; 
// Clear the controler history and the controller output
PIDInit(&AnglePIDstruct); 
//Derive the a,b, & c coefficients from the Kp, Ki & Kd
PIDCoeffCalc(&AngleKCoeffs[0], &AnglePIDstruct); 
}

void Orientation(void)	// [23]
{
	int DeltaVel;	// difference in speed between the wheels to rotate
	int RealVel;	// VelDesM after reduction controlled by Dist PID
	float Error;	
	
	ORIENTATION_FLAG = 0; // it will be restarted by DeadReckoning()
	
	if (ThetaDes < 0) ThetaDes += TWOPI;	// keep angle value positive
	if (ThetaMes < 0) ThetaMes += TWOPI;
	Error = ThetaMes - ThetaDes;
	if (Error > PI)	// search for the best direction to correct error [23a]
	{
		Error -= TWOPI;
	}
	else if (Error < -PI)
	{
		Error += TWOPI;
	}

	ANGLE_PID_DES = 0; 				// ref value translated to 0 [23c]
	ANGLE_PID_MES = Q15(Error/PI);	// current error [23b]
	PID(&AnglePIDstruct);
	
	DeltaVel = (ANGLE_PID_OUT >> 7);// MAX delta in int = 256 [23d]
	RealVel = VelDesM - VelDecr;	// [24d]
	VelDes[R] = RealVel - DeltaVel;	// [23e]
	VelDes[L] = RealVel + DeltaVel;

	if (VelDes[R] > MAX_ROT_SPEED)
	{
		VelDes[R] = MAX_ROT_SPEED;
		VelDes[L] = MAX_ROT_SPEED + (DeltaVel << 1);
	}
	else if (VelDes[R] < -MAX_ROT_SPEED)
	{
		VelDes[R] = -MAX_ROT_SPEED;
		VelDes[L] = -MAX_ROT_SPEED + (DeltaVel << 1);
	}
	else if (VelDes[L] > MAX_ROT_SPEED)
	{
		VelDes[L] = MAX_ROT_SPEED;
		VelDes[R] = MAX_ROT_SPEED - (DeltaVel << 1);
	}
	else if (VelDes[L] < -MAX_ROT_SPEED)
	{
		VelDes[L] = -MAX_ROT_SPEED;
		VelDes[R] = -MAX_ROT_SPEED - (DeltaVel << 1);
	}

	SendVelStatus = 1;	// Start sending VelDes[x] to MCs
}

unsigned char IncrCircPtr(void) // [6d]
{
	TmpPtr ++;								// next byte
	if (TmpPtr >= MAX_RX_BUFF) TmpPtr=0;	// reset circular queue
	return TmpPtr;
}

unsigned char IncrCircPtr2(void) // [6d]
{
	TmpPtr2 ++;								// next byte
	if (TmpPtr2 >= MAX_RX_BUFF) TmpPtr2=0;	//reset circular queue
	return TmpPtr2;
}

void DeadReckoning(void) // [21]
{
	float CosNow;		// current value for Cos
	float SinNow;		// current value for Sin
	float DSpace;		// delta traveled distance by the robot
	float DTheta;		// delta rotation angle
	float DPosX;		// delta space on X axis
	float DPosY;		// delta space on Y axis
	float SaMinusSb;
	float SrPlusSl;
	float Radius;
	
	int Xnorm;	// X index in 0-49 range
 	int Xindx;	// X index in 0-24 range
 	int Yindx;	// Y index in 0-49 range
 	div_t z;	// for div function
 			
	SPACE_FLAG=0;	// [21c]
	ODO_FLAG=0;
	
	Spmm[R]=SpTick[R]*Ksp[R];	// distance of right wheel in mm
	Spmm[L]=SpTick[L]*Ksp[L];	// distance of left wheel in mm

/**debug**
	TickR=TickR+SpTick[R];
	TickL=TickL+SpTick[L];
**debug*/
	
	#ifdef geographic			// [21aa]
		SaMinusSb=Spmm[L]-Spmm[R];
	#else
		SaMinusSb=Spmm[R]-Spmm[L];
	#endif
	
	SrPlusSl=Spmm[R]+Spmm[L];
	if (fabs(SaMinusSb) <= SPMIN)
	{// traveling in a nearly straight line [21a]
		DSpace=Spmm[R];
		
		#ifdef geographic			// [21aa]
			DPosX=DSpace*SinPrev;
			DPosY=DSpace*CosPrev;
		#else
			DPosX=DSpace*CosPrev;
			DPosY=DSpace*SinPrev;
		#endif

	}
	else if (fabs(SrPlusSl) <= SPMIN)
	{// pivoting around vertical axis without translation [21a]
		DTheta=SaMinusSb/Axle;
		ThetaMes=fmodf((ThetaMes+DTheta),TWOPI);//current orient. in 2PI range
		CosPrev=cosf(ThetaMes);	// for the next cycle
		SinPrev=sinf(ThetaMes);
		DPosX=0;
		DPosY=0;
		DSpace=0;
	}
	else
	{// rounding a curve	
		DTheta=SaMinusSb/Axle;
		ThetaMes=fmodf((ThetaMes+DTheta),TWOPI);//current orient. in 2PI range
		CosNow=cosf(ThetaMes);
		SinNow=sinf(ThetaMes);
		DSpace=SrPlusSl/2;
		Radius = (SEMI_AXLE)*(SrPlusSl/SaMinusSb);
		
		#ifdef geographic			// [21aa]
			DPosX=Radius*(CosPrev-CosNow);	
			DPosY=Radius*(SinNow-SinPrev);
		#else
			DPosX=Radius*(SinNow-SinPrev);
			DPosY=Radius*(CosPrev-CosNow);
		#endif		

		CosPrev=CosNow;		// to avoid re-calculation on the next cycle
		SinPrev=SinNow;
	}

	Space += DSpace;	// total traveled distance
	PosXmes += DPosX;		// current position
	PosYmes += DPosY;
	
	// field mapping [21b]
	Xnorm=abs((__builtin_modsd((PosXmes+5000),5000))/100);	// index in the range 0-49
	Yindx=abs((__builtin_modsd((PosYmes+5000),5000))/100);

	z=div(Xnorm,2);		// index in the range 0-24 and remainder 0-1
	Xindx=z.quot;
	
	if ((Xnorm != XnormPrev) || (Yindx != YindxPrev)) //only when cell changes
	{
		if (z.rem)
		{// increment cell value only if < 10
			if (MapXY[Xindx][Yindx].High < 10) MapXY[Xindx][Yindx].High ++;
		}
		else
		{
			if (MapXY[Xindx][Yindx].Low < 10) MapXY[Xindx][Yindx].Low ++;		
		}
		XnormPrev=Xnorm;
		YindxPrev=Yindx;		
	}
	ORIENTATION_FLAG = 1; // position coordinates computed, Angle PID can start
}

void AskParamCall(void)
{
	if (AskParamStatus == 1) // [20]
	{
		IdTx = ID_R;
		TX_EN2 = 0;
		Nop();
		TX_EN1 = 1;
	}
	if (AskParamStatus == 2)
	{
		IdTx = ID_L;
		TX_EN1 = 0;
		Nop();	
		TX_EN2 = 1;
	}
	switch (AskParam)
	{
		case 1:
			TxParameters(IdTx,'P',0);  // position value request [16]		
		break;
		
		case 4:
			TxParameters(IdTx,'A',0);  // all parameters request [16]		
		break;

		case 2:
			TxParameters(IdTx,'V',0);  // speed parameter request [16]		
		break;

		case 3:
			TxParameters(IdTx,'C',0);  // motor current request [16]		
		break;
	}	
	ASK_PARAM_FLAG = 0; // [20b]
}

void TxParameters(unsigned char Id,unsigned char TxCmd,int TxCmdLen)	// [18]
{
	int TxCount=0;
	UartTxBuff[0] = '@';		// Header
	UartTxBuff[1] = Id;			// Id
	UartTxBuff[2] = TxCmd;		// Cmd
	UartTxBuff[3] = TxCmdLen +1;	// CmdLen
	if (TxCmdLen)	// at least one parameter to send
	{
		for (TxCount = 0; TxCount < TxCmdLen; TxCount ++)
		{
			UartTxBuff[TxCount + 4] = UartTmpBuff[TxCount];
		}
	}
	UartTxBuff[TxCount + 4] = UartChkSum(UartTxBuff,TxCount + 4);
	UartTxBuffSize = TxCount + 5;
	TX_FLAG =1;	// invia il buffer
}

void Tx2Parameters(char TxCmd,int TxCmdLen)	// [18a]
{
	int TxCount;
	
	UartTx2Buff[0] = '@';			// Header
	UartTx2Buff[1] = ID_A;			// Id broadcast
	UartTx2Buff[2] = TxCmd;			// Cmd
	UartTx2Buff[3] = TxCmdLen +1;	// CmdLen
	if (TxCmdLen)	// at least one parameter to send
	{
		for (TxCount = 0; TxCount < TxCmdLen; TxCount ++)
		{
			UartTx2Buff[TxCount + 4] = UartTmp2Buff[TxCount];
		}
	}
	UartTx2Buff[TxCount + 4] = UartChkSum(UartTx2Buff,TxCount + 4);
	UartTx2BuffSize = TxCount + 5;
	TX2_FLAG =1;	// sends buffer
}

void UartTx(void)	// [6a]
{
UartTxCntr = 1;	// initializes array to send
TX_FLAG = 0;
TX_BUFF_FULL = 1;

// Enable RX on MC according to the Id
if (UartTxBuff[1] == ID_A)
{
	CS1 = 1;	// broadcast, both MCs enabled
	Nop();
	CS2 = 1;
}
else if (UartTxBuff[1] == ID_R)
{
	CS2 = 0;
	Nop();
	CS1 = 1;
}
else if (UartTxBuff[1] == ID_L)
{
	CS1 = 0;
	Nop();
	CS2 = 1;
}

// first INT occurs and everything is managed by ISR
WriteUART1(UartTxBuff[UartTxCntr-1]); // sends first character
}

void UartTx2(void)	// [6d]
{
UartTx2Cntr = 1;	// initializes array to send
TX2_FLAG = 0;
TX2_BUFF_FULL = 1;
// first INT happens and everything is managed by ISR
WriteUART2(UartTx2Buff[UartTx2Cntr-1]); // sends first character
}

void UartRx(void)	// [6b]
{	
	if (UartRxStatus < 0)	// if error, RX is terminated
	{
		UartRxError(UartRxStatus);		   	
	}
	else					// 	otherwise it analyzes status 
	{					
		switch (UartRxStatus)
		{     
   			case 0:	// idle
   	    		if (UartRxBuff[UartRxPtrOut] == HEADER)
   	    		{
	   	    		UartRxStatus = 1;	// next status
	   	    		UartRxPtrStart = UartRxPtrOut;
	   	    		ChkSum = ChkSum + UartRxBuff[UartRxPtrOut];
					UartRxPtrOut ++;	// next byte
	   	    	}
	   	    	else					// out of command sequence -> error
	   	    	{
					UartRxError(-5);		   	
				}
			break;
		
			case 1:	// header received
   	    		IdRx = (UartRxBuff[UartRxPtrOut]); // motor Id
   	    		IdIndx = IdRx -1;		// array index
   	     	 	UartRxStatus = 2;		// next status
   	     	 	ChkSum = ChkSum + UartRxBuff[UartRxPtrOut];
				UartRxPtrOut ++;		// next byte
			break;
		
			case 2:	// command received
   	     	 	UartRxStatus = 3;		// next status
   	     	 	UartRxCmd=UartRxBuff[UartRxPtrOut];
   	     	 	ChkSum = ChkSum + UartRxBuff[UartRxPtrOut];
				UartRxPtrOut ++;		// next byte
			break;
		
			case 3:	// command length received
				// calculating end pointer of command string
				UartRxPtrEnd=UartRxBuff[UartRxPtrOut]+RX_HEADER_LEN+
					UartRxPtrStart;
				if (UartRxPtrEnd >= MAX_RX_BUFF)UartRxPtrEnd -= MAX_RX_BUFF;
				if (UartRxPtrEnd >= MAX_RX_BUFF)	// still bigger?
				{
					UartRxError(-11);	// queue overflow	
				}
				else
				{
   	     	 		UartRxStatus = 4;		// next status
   	     	 		ChkSum = ChkSum + UartRxBuff[UartRxPtrOut];
   	     	 		UartRxPtrData=UartRxPtrOut;// pointer to last header byte
					UartRxPtrOut ++;		// next byte
				}
			break;
			
			case 4:	// waiting for command end
   	      		if (UartRxPtrOut == UartRxPtrEnd)
   	      		{	// checksum and Id OK ?		   	    
	   	      		if (ChkSum == UartRxBuff[UartRxPtrOut] && (IdRx  == IdTx))
	   	      		{
	   	      			UartRxStatus=99;// enables command parser  
						UartRxPtrOut ++;	// next byte
		   	      	}
		   	      	else
		   	      	{
			   	    	UartRxError(-1); // checksum error	
			   	    }
			   	  	ChkSum=0;
   	      		}
   	      		else
   	      		{
   	      			ChkSum = ChkSum + UartRxBuff[UartRxPtrOut];
					UartRxPtrOut ++;	// next byte
			   	}
			break;
		
			default:// error: not a known status
				UartRxError(-6);				   	
			break;
		} // switch end
	}	// if end
		
	if (UartRxPtrOut >= MAX_RX_BUFF) UartRxPtrOut=0;//reset circular queue[6f]
}

void UartRx2(void)	// [6d]
{	
	if (UartRx2Status < 0)	// if error, RX is terminated
	{
		UartRx2Error(UartRx2Status);		   	
	}
	else					// 	otherwise it analyzes status 
	{					
		switch (UartRx2Status)
		{     
   			case 0:	// idle
   	    		if (UartRx2Buff[UartRx2PtrOut] == HEADER)
   	    		{
	   	    		UartRx2Status = 1;	// next status
	   	    		UartRx2PtrStart = UartRx2PtrOut;
	   	    		ChkSum2 = ChkSum2 + UartRx2Buff[UartRx2PtrOut];
	   	    		UartRx2PtrOut ++;	// next byte
	   	    	}
	   	    	else					// out of sequence command -> error
	   	    	{
					UartRx2Error(-5);		   	
				}
			break;
		
			case 1:	// header received
   	    		IdRx2 = (UartRx2Buff[UartRx2PtrOut]);// MC address
   	    		if (IdRx2 != 0)	// only if not broadcast 
   	    		{
   	    			IdIndx2 = IdRx2 -1;		// array index
   	    		}
   	    		else
   	    		{
	   	    		IdIndx2 = 0;
	   	    	}
   	     	 	UartRx2Status = 2;		// next status
   	     	 	ChkSum2 = ChkSum2 + UartRx2Buff[UartRx2PtrOut];
	   	    	UartRx2PtrOut ++;	// next byte
			break;
		
			case 2:	// command received
   	     	 	UartRx2Status = 3;		// next status
   	     	 	UartRx2Cmd=UartRx2Buff[UartRx2PtrOut];
   	     	 	ChkSum2 = ChkSum2 + UartRx2Buff[UartRx2PtrOut];
	   	    	UartRx2PtrOut ++;	// next byte
			break;
		
			case 3:	// command length received
				// calculating end pointer of command string
				UartRx2PtrEnd=UartRx2Buff[UartRx2PtrOut]+RX_HEADER_LEN+
					UartRx2PtrStart;
				if (UartRx2PtrEnd >= MAX_RX_BUFF)UartRx2PtrEnd -= MAX_RX_BUFF;
				if (UartRx2PtrEnd >= MAX_RX_BUFF)	// still bigger?
				{
					UartRx2Error(-11); //  queue overflow	
				}
				else
				{
   	     	 		UartRx2Status = 4;		// next status
   	     	 		ChkSum2 = ChkSum2 + UartRx2Buff[UartRx2PtrOut];
   	     	 		UartRx2PtrData=UartRx2PtrOut;// pointer to last header byte
	   	    		UartRx2PtrOut ++;	// next byte
	   	    	}
			break;
			
			case 4:	// waiting for command end
   	      		if (UartRx2PtrOut == UartRx2PtrEnd)
   	      		{	
	   	      		if (ChkSum2 == UartRx2Buff[UartRx2PtrOut])// checksum OK?		   	    		   	    
	   	      		{
	   	      			UartRx2Status=99;	// enables command parser
	   	    			UartRx2PtrOut ++;	// next byte
		   	      	}
		   	      	else
		   	      	{
			   	    	UartRx2Error(-1); // checksum error	
			   	    }
			   	  	ChkSum2=0;
   	      		}
  	      		else
   	      		{
   	      			ChkSum2 = ChkSum2 + UartRx2Buff[UartRx2PtrOut];
	   	    		UartRx2PtrOut ++;	// next byte
			   	}
			break;
		
			default:// error: not a known status
				UartRx2Error(-6);				   	
			break;
		} // fine switch
	}	// fine if
		
	if (UartRx2PtrOut>=MAX_RX_BUFF) UartRx2PtrOut=0;//reset circular queue[6f]
}

unsigned char UartChkSum (unsigned char *Buff,unsigned int BuffSize)	// [17]
{
	unsigned char ChkSum=0;	// checksum
	int ChkIndx;
	for (ChkIndx = 0; ChkIndx < BuffSize; ChkIndx ++)
	{
		ChkSum = ChkSum + Buff[ChkIndx];
	}
			
/*	#ifdef debug_SA	// [15f]
		return '@';
	#else
		return (ChkSum);
	#endif */	   	

	return (ChkSum);      	
}	
	   	      	
void UartRxError(int Err)	// RX error occured
{
	LED2=1;					// turns on error LED
	UartRxPtrOut=0;			// flush circular queue
	UartRxPtrIn=0;
	UartRxPtrEnd=0;
	UartRxStatus=0; 
	ChkSum=0;
	TX_BUFF_FULL = 0;		// TX buffer empty
	TX_PENDING = 0;			// no TX sequence pending
	AskParamStatus = 0;	
	while (U1STAbits.URXDA)	// flush USART RX buffer
	{
		ReadUART1();
	}

	// **debug**
//	ErrNo[IdIndx][abs(Err)]++;	// error log
	// **debug**
	
//	CloseUART1();			// reset UART1
//	OpenUART1(U1MODEvalue, U1STAvalue, Ubrg);// reset UART1
}

void UartRx2Error(int Err)	// RX error occured
{
	LED2=1;					// turns on error LED
	UartRx2PtrOut=0;		// flush circular queue
	UartRx2PtrIn=0;
	UartRx2PtrEnd=0;
	UartRx2Status=0; 
	ChkSum2=0;
	while (U2STAbits.URXDA)	// flush USART RX buffer
	{
		ReadUART2();
	}
	// **debug**
//	ErrNo2[abs(Err)]++;		// error log
	// **debug**
}

void CmdHalt(void)	// [16]
{// immediate Halt without decelerating ramp for both motors
	TX_PENDING = 1;	// TX sequence started
	TxParameters(0,'H',0);  		
}

void CmdWriteVel(unsigned char Id,int Vel)	// [16]
{// setting reference speed as mm/s (-999 +999)
	TX_PENDING = 1;	// TX sequence started
	UartTmpBuff[0]=Vel>>8; 
	UartTmpBuff[1]=Vel;
	TxParameters(Id,'W',2);  		
}

void CmdWriteId(unsigned char Id, unsigned char Id1)	// [16a]
{// Id designation
	TX_PENDING = 1;	// TX sequence started
	UartTmpBuff[0]=Id1; 
	TxParameters(Id,'I',1);  		
}

void CmdWriteParK(unsigned char Id, unsigned char Indx) // [16]
{// setting of Kp, Ki, Kd coefficients
	TX_PENDING = 1;	// TX sequence started
	UartTmpBuff[0]=Kpid[0][Indx]>>8; 
	UartTmpBuff[1]=Kpid[0][Indx];
	UartTmpBuff[2]=Kpid[1][Indx]>>8; 
	UartTmpBuff[3]=Kpid[1][Indx];
	UartTmpBuff[4]=Kpid[2][Indx]>>8; 
	UartTmpBuff[5]=Kpid[2][Indx];
	TxParameters(Id,'K',6);		
}

void CmdWriteParKvel(unsigned char Id, unsigned char Indx) // [16]
{// setting Kvel coefficient
	TX_PENDING = 1;	// TX sequence started
	UartTmpBuff[0]=Kvel[Indx]>>24; 
	UartTmpBuff[1]=Kvel[Indx]>>16;
	UartTmpBuff[2]=Kvel[Indx]>>8; 
	UartTmpBuff[3]=Kvel[Indx];
	TxParameters(Id,'L',4);		
}

void Parser (void)	// [16]
{
	char ParserFlag=0;
	TmpPtr = UartRxPtrData; // last byte of header
	switch (UartRxCmd)
	{
		case 'P':		// traveled distance = int -> 2byte
			 	SpTick[IdIndx]=(UartRxBuff[IncrCircPtr()] << 8) + 
					   	  (UartRxBuff[IncrCircPtr()]);
				SPACE_FLAG=1;	// [21c]
		break;
		
		case 'A':		// all parameters
				// measured speed value = Int -> 2 byte
				VelMes[IdIndx]=(UartRxBuff[IncrCircPtr()] << 8) + 
					   		 (UartRxBuff[IncrCircPtr()]);
				// motor current = int -> 2byte
			 	Curr[IdIndx]=(UartRxBuff[IncrCircPtr()] << 8) + 
					   	  (UartRxBuff[IncrCircPtr()]);
				// traveled distance = int -> 2byte
			 	SpTick[IdIndx]=(UartRxBuff[IncrCircPtr()] << 8) + 
					   	  (UartRxBuff[IncrCircPtr()]);	
				SPACE_FLAG=1;	// [21c]
		break;
		
		case 'V':		// measured speed value = Int -> 2 byte
				VelMes[IdIndx]=(UartRxBuff[IncrCircPtr()] << 8) + 
					   		 (UartRxBuff[IncrCircPtr()]);
		break;
		
		case 'C':		// motor current = int -> 2byte
			 	Curr[IdIndx]=(UartRxBuff[IncrCircPtr()] << 8) + 
					   	  (UartRxBuff[IncrCircPtr()]);
		break;
		
				
		default:
			UartRxError(-7); // error: not a known command
			ParserFlag=1;
		break;
	}
	
	if (!ParserFlag)		  // known command
	{
		if (AskParamStatus == 1)	// received from MC 1
		{
			AskParamStatus = 2;		// waits RX from MC 2
			ASK_PARAM_FLAG=1;		// re-enables parameters request
		}
		else if (AskParamStatus == 2)	// received from MC 2 
		{	
			AskParamStatus = 0;		// end of cycle
			ASK_PARAM_FLAG=0;		// it will be set again by ISR 1ms
			TX_EN2 = 0;
			if (SPACE_FLAG) ODO_FLAG=1;	// [21c]
			TX_EN1 = 0;
		}
	}
	UartRxStatus = 0;	
}

void Parser2 (void)	// [16b]
{
	int ParserCount;	// parser index
	TmpPtr2 = UartRx2PtrData; // last byte of header
	unsigned int Ktmp;	// temp for PID coefficients
	int Ptmp;			// temp for position values
	int Alpha;			// rotation angle in degrees
	long AxleLong;		// temp for Axle 
	long KspLong;		// temp for Ksp

if (UartRx2Cmd != '*')	// [28]
{
	ResetCount = 0;
}

	switch (UartRx2Cmd)
	{		
		case '*':		// Board reset [28]
				if (ResetCount < 3)
				{
					ResetCount ++;
				}
				else
				{
					SET_CPU_IPL( 7 ); // disable all user interrupts
					DelayN1ms(200);   // waits for the MCs reset
					asm("RESET");
				}
		break;
		
		case 'R':		// Firmware version request
				// Sende string 'Ver'
				for (i=0; i<27; i++)
				{
			 		UartTmp2Buff[i]=Ver[i]; 
				}
				Tx2Parameters('R',26);
		break;

		case 'D':		// setting reference coord. X, Y computing distance
						// [24] Mode C
			// High Byte * 256 + Low Byte
			Ptmp = (UartRx2Buff[IncrCircPtr2()] << 8) + 
							(UartRx2Buff[IncrCircPtr2()]); // Dist
			PosXdes = PosXmes + (Ptmp * sin(ThetaDes));
			PosYdes = PosYmes + (Ptmp * cos(ThetaDes));
			DIST_ENABLE_FLAG = 1;	// enable distance computing [24a]
		break;
		
		case 'x':		// setting reference coordinates X, Y in mm
						// [24] Mode B
			// High Byte * 256 + Low Byte
			PosXdes = (float)((UartRx2Buff[IncrCircPtr2()] << 8) + 
							(UartRx2Buff[IncrCircPtr2()]));
			PosYdes = (float)((UartRx2Buff[IncrCircPtr2()] << 8) + 
							(UartRx2Buff[IncrCircPtr2()]));
			DIST_ENABLE_FLAG = 1;	// enable distance computing [24a]
		break;

		case 'O':		// setting ref. orientation angle in degrees (absolute)
						// [24] Mode A
			// High Byte * 256 + Low Byte
			Ptmp = (UartRx2Buff[IncrCircPtr2()] << 8) + 
							(UartRx2Buff[IncrCircPtr2()]);
			ThetaDes = Ptmp * DEG2RAD;
			DIST_ENABLE_FLAG = 0;	// disable distance computing [24a]
			VelDecr = 0;			// [24d]
		break;

		case 'o':		// setting reference orientation angle in degrees
						// as a delta of the current orientation (relative) 
						// [24] Mode A
			// High Byte * 256 + Low Byte
			Ptmp = (UartRx2Buff[IncrCircPtr2()] << 8) + 
							(UartRx2Buff[IncrCircPtr2()]);
			ThetaDes += Ptmp * DEG2RAD;
			DIST_ENABLE_FLAG = 0;	// disable distance computing [24a]
			VelDecr = 0;			// [24d]
		break;
		
		case 'H':		// immediate Halt without decelerating ramp
			CmdHalt();
			DIST_ENABLE_FLAG = 0;	// disable distance computing [24a]
			VelDecr = 0;			// [24d]
		break;
	
		case 'W':		// setting reference speed (as mm/s) 
			// High Byte * 256 + Low Byte
			VelDesM = (UartRx2Buff[IncrCircPtr2()] << 8) + 
							(UartRx2Buff[IncrCircPtr2()]);
		break;
		
		case 'K':		// settings Speed PID coefficients
			for (ParserCount=0; ParserCount < 5; ParserCount+=2)
			{
				// High Byte * 256 + Low Byte
				Ktmp = (UartRx2Buff[IncrCircPtr2()] << 8) + 
							(UartRx2Buff[IncrCircPtr2()]);
				if (Ktmp > 999) Ktmp = 999; // range check
				Kpid[ParserCount/2][IdIndx2] = Ktmp;
			}
			CmdWriteParK(IdRx2,IdIndx2);
			EepromWrite(0);	
		break;

		case 'L':		// settings Kvel coefficient
			Kvel[IdIndx2] = ((long)UartRx2Buff[IncrCircPtr2()] << 24) +
						  ((long)UartRx2Buff[IncrCircPtr2()] << 16) +
						  ((long)UartRx2Buff[IncrCircPtr2()] << 8) +
						  ((long)UartRx2Buff[IncrCircPtr2()]);
			CmdWriteParKvel(IdRx2,IdIndx2);
			EepromWrite(0);	
		break;
		
		case 'M':		// Mechanical costants: Axle size, Ksp1, Ksp2 [29]
				AxleLong = ((long)UartRx2Buff[IncrCircPtr2()] << 24) +
						  ((long)UartRx2Buff[IncrCircPtr2()] << 16) +
						  ((long)UartRx2Buff[IncrCircPtr2()] << 8) +
						  ((long)UartRx2Buff[IncrCircPtr2()]);
				Axle = (float)(AxleLong/10000);
				
				KspLong = ((long)UartRx2Buff[IncrCircPtr2()] << 24) +
						  ((long)UartRx2Buff[IncrCircPtr2()] << 16) +
						  ((long)UartRx2Buff[IncrCircPtr2()] << 8) +
						  ((long)UartRx2Buff[IncrCircPtr2()]);
				Ksp[0] = (float)(KspLong/1000000);
				
				KspLong = ((long)UartRx2Buff[IncrCircPtr2()] << 24) +
						  ((long)UartRx2Buff[IncrCircPtr2()] << 16) +
						  ((long)UartRx2Buff[IncrCircPtr2()] << 8) +
						  ((long)UartRx2Buff[IncrCircPtr2()]);
				Ksp[1] = (float)(KspLong/1000000);
				EepromWrite(1);	
		break;

		case 'k':		// settings PID coefficients for AnglePid
			for (ParserCount=0; ParserCount < 5; ParserCount+=2)
			{
				// High Byte * 256 + Low Byte
				Ktmp = (UartRx2Buff[IncrCircPtr2()] << 8) + 
							(UartRx2Buff[IncrCircPtr2()]);
				if (Ktmp > 9999) Ktmp = 9999; // range check
				AngleKCoeffs[ParserCount/2] = Q15((float)(Ktmp)/10000);
			}
			InitAnglePid();
			EepromWrite(0);	
		break;

		case 'J':		// settings PID coefficients for DistPid
			for (ParserCount=0; ParserCount < 5; ParserCount+=2)
			{
				// High Byte * 256 + Low Byte
				Ktmp = (UartRx2Buff[IncrCircPtr2()] << 8) + 
							(UartRx2Buff[IncrCircPtr2()]);
				if (Ktmp > 9999) Ktmp = 9999; // range check
				DistKCoeffs[ParserCount/2] = Q15((float)(Ktmp)/10000);
			}
			InitDistPid();
			EepromWrite(0);	
		break;

		case 'A':		// all parameters
				// VelMes = Int -> 2 byte (mm/s)
				Ptmp = (VelMes[R] + VelMes[L]) >> 1;	// average speed
			 	UartTmp2Buff[0]=Ptmp>>8; 
				UartTmp2Buff[1]=Ptmp;
				// Curr = int -> 2byte (mA)
			 	UartTmp2Buff[2]=Curr[IdIndx2]>>8;
				UartTmp2Buff[3]=Curr[IdIndx2];
				// PosXmes rounded in a Int -> 2 byte (mm)
				Ptmp = FLOAT2INT(PosXmes);
			 	UartTmp2Buff[4]=Ptmp>>8; 
				UartTmp2Buff[5]=Ptmp;
				// PosYmes rounded in a Int -> 2 byte (mm)
				Ptmp = FLOAT2INT(PosYmes);
			 	UartTmp2Buff[6]=Ptmp>>8; 
				UartTmp2Buff[7]=Ptmp;
				// ThetaMes rounded in a Int -> 2 byte (degrees)
				Ptmp = ThetaMes * RAD2DEG;
				Alpha = FLOAT2INT(Ptmp);
			 	UartTmp2Buff[8]=Alpha>>8; 
				UartTmp2Buff[9]=Alpha;
				Tx2Parameters('A',10);  
		break;
		
		case 'X':		// bot position coordinates request
				/* PosXmes, PosYmes and ThetaMes are float
				   for displaying is enough to round them in a Int */
				Ptmp = FLOAT2INT(PosXmes);
			 	UartTmp2Buff[0]=Ptmp>>8; 
				UartTmp2Buff[1]=Ptmp;
				Ptmp = FLOAT2INT(PosYmes);
			 	UartTmp2Buff[0]=Ptmp>>8; 
				UartTmp2Buff[1]=Ptmp;
				Ptmp = FLOAT2INT(ThetaMes);
			 	UartTmp2Buff[0]=Ptmp>>8; 
				UartTmp2Buff[1]=Ptmp;
				Tx2Parameters('X',6);
		break;
				
		case 'V':		// speed value
				// VelMes = Int -> 2 byte (mm/s)
				Ptmp = (VelMes[R] + VelMes[L]) >> 1;	// average speed
			 	UartTmp2Buff[0]=Ptmp>>8; 
				UartTmp2Buff[1]=Ptmp;
				Tx2Parameters('V',2);
		break;
		
		case 'C':		// motor current
				// Curr = int -> 2byte
			 	UartTmp2Buff[0]=Curr[IdIndx2]>>8;
				UartTmp2Buff[1]=Curr[IdIndx2];
				Tx2Parameters('C',2);
		break;
		
		case 'P':		// position value
				// SpTick = int -> 2byte
			 	UartTmp2Buff[0]=SpTick[IdIndx2]>>8;
				UartTmp2Buff[0]=SpTick[IdIndx2];
				Tx2Parameters('P',2); 
		break;
		
		default:
			UartRx2Error(-7); //	error: not a known command
		break;
	}
	
	UartRx2Status = 0;
}

void MC_Setting(void)	// both motor controllers configuration [16a]
{
DelayN10us(2);			// waits for stable clock on MCs
CS2 = 0;
Nop();
CS1 = 1;				// starts MC 1 program	
DelayN10us(3);			// waits for MC 1 inizialization
CmdWriteId(ID_I, ID_R);	// sets MC R ID, changing it from initial ID

do // transmission sequence pending
{
	if (TX_FLAG)	UartTx();
} while (TX_BUFF_FULL);

DelayN10us(3);

CS1 = 0;
Nop();
CS2 = 1;				// starts MC 2 program	
DelayN10us(3);			// waits for MC 2 inizialization
CmdWriteId(ID_I, ID_L);	// sets MC L ID, changing it from initial ID

do // transmission sequence pending
{
	if (TX_FLAG)	UartTx();
} while (TX_BUFF_FULL);
DelayN10us(3);			// waits for command parsing
CmdWriteParK(ID_R,R);	// sets Kp, Ki, Kd on MCs
do // transmission sequence pending
{
	if (TX_FLAG)	UartTx();
} while (TX_BUFF_FULL);
DelayN10us(20);			// waits for command parsing
CmdWriteParK(ID_L,L);	// sets Kp, Ki, Kd on MCs
do // transmission sequence pending
{
	if (TX_FLAG)	UartTx();
} while (TX_BUFF_FULL);
DelayN10us(20);			// waits for command parsing
CmdWriteParKvel(ID_R,R);	// sets KVel on MCs
do // transmission sequence pending
{
	if (TX_FLAG)	UartTx();
} while (TX_BUFF_FULL);
DelayN10us(20);			// waits for command parsing
CmdWriteParKvel(ID_L,L);	// sets KVel on MCs
do // transmission sequence pending
{
	if (TX_FLAG)	UartTx();
} while (TX_BUFF_FULL);
DelayN10us(20);			// waits for command parsing
}

void DelayN1ms(int n)
{
	int ms;
	for (ms = 0; ms < n; ms ++)
	{
		DelayN10us(100);
	}
}

void DelayN10us(int n) // [22]
{
	int DelayCount;
	for (DelayCount = 0; DelayCount < (42 * n); DelayCount ++);	
}
/*---------------------------------------------------------------------------*/
/* Interrupt Service Routines                                                */
/*---------------------------------------------------------------------------*/
void _ISR_PSV _T1Interrupt(void)	// Timer 1 [13]
{
	_T1IF=0;   		// interrupt flag reset
	HRT_BT = 1;		// Hartbeat 
	Blink ++;		// index for heartbeat LED blinking (6 cycles)
	HRT_BT = 0;		// pulse is high for 6 cycles (200us) every 1ms
	ClrWdt();		// [1]
	if (AskParam)	// [20a]
	{
		AskParamTimer1 ++;
		if (AskParamTimer1 >= ASK_PARAM_TMO)
		{
			ASK_PARAM_FLAG = 1; // it's time to ask parameters
			if (AskParamStatus)
			{
				UartRxError(-10);	// MC timeout error [20c]
			}
			AskParamStatus = 1; 
			AskParamTimer2 ++;
			if (AskParamTimer2 >= ASK_PARAM_ALL_TMO)
			{
				AskParamTimer2 = 0;
				AskParam = 4; // requests all parameters every n cycles
				DIST_TIMER_FLAG=1;//compute distance with the same period [24b]
				IdleSample ++; // [25]
			}
			else
			{
				AskParam = 1;
			}
			AskParamTimer1 = 0;
		}
	}
}

void _ISR_PSV _U1RXInterrupt(void)	// UART1 RX [6b]
{
	_U1RXIF = 0; 	// interrupt flag reset
	
	// UART errors, still parsing command or buffer overflow ?
	if (!OVERRUN_ERROR && !FRAME_ERROR && UartRxStatus!=99)	
	{
		UartRxBuff[UartRxPtrIn] = ReadUART1();	// fills RX buffer
		UartRxPtrIn ++;							// next byte
		if (UartRxPtrIn>=MAX_RX_BUFF) UartRxPtrIn=0;// circular queue[6d]
		UartRxTmr = UART_RX_TIME_OUT;	// timeout reset
	}
	else
	{
		// indicates the error kind [6b]
		if (OVERRUN_ERROR)
		{
			UartRxStatus = -4;
			OVERRUN_ERROR = 0;	// error reset & flush RX buffer
		}
		if (FRAME_ERROR) UartRxStatus = -3;
		if (UartRxStatus==99) UartRxStatus = -9;
		UartRxPtrIn ++;	// start RX routine to analyze error
		if (UartRxPtrIn>=MAX_RX_BUFF) UartRxPtrIn=0;// circular queue[6d]
		UartRxTmr = UART_RX_TIME_OUT;	// timeout reset
	}
}

void _ISR_PSV _U2RXInterrupt(void)	// UART2 RX [6d]
{
	_U2RXIF = 0; 	// interrupt flag reset
	
	// UART errors, still parsing command or buffer overflow ?
	if (!OVERRUN_ERROR2 && !FRAME_ERROR2 && UartRx2Status!=99)	
	{
		UartRx2Buff[UartRx2PtrIn] = ReadUART2();	// fills RX buffer
		UartRx2PtrIn ++;							// next byte
		if (UartRx2PtrIn>=MAX_RX_BUFF)UartRx2PtrIn=0;//reset circular queue[6d]
		UartRx2Tmr = UART_RX2_TIME_OUT;	// timeout reset
	}
	else
	{
		// indicates the error kind [6b]
		if (OVERRUN_ERROR2) 
		{
			UartRx2Status = -4;
			OVERRUN_ERROR2 = 0;	// error reset & flush RX2 buffer
		}
		if (FRAME_ERROR2) UartRx2Status = -3;
		if (UartRx2Status==99) UartRx2Status = -9;
		UartRx2PtrIn ++;	// start RX routine to analyze error
		if (UartRx2PtrIn>=MAX_RX_BUFF)UartRx2PtrIn=0;//reset circular queue[6d]
		UartRx2Tmr = UART_RX_TIME_OUT;	// timeout reset
	}
}

void _ISR_PSV _U1TXInterrupt(void)	// UART1 TX [6a]
{
	_U1TXIF = 0;	// interrupt flag reset
	if (UartTxCntr < UartTxBuffSize)
	{
		WriteUART1(UartTxBuff[UartTxCntr]);
		UartTxCntr++;
	}
	else
	{
		TX_BUFF_FULL = 0;	// buffer empty 
		TX_PENDING = 0;		// TX sequence over
	}
}

void _ISR_PSV _U2TXInterrupt(void)	// UART2 TX [6d]
{
	_U2TXIF = 0;	// interrupt flag reset
	if (UartTx2Cntr < UartTx2BuffSize)
	{
		WriteUART2(UartTx2Buff[UartTx2Cntr]);
		UartTx2Cntr++;
	}
	else
	{// waits for transmission end to disable UART
//		TX2_FLAG = 2; //########################	
	}
}

void _ISR_PSV _NVMInterrupt(void)	// EEPROM [30]
{
	_NVMIF = 0;	// interrupt flag reset
	if (EEPROM_WRITE_STATUS_FLAG)	// status = write	
	{
		_write_eedata_row(EE_addr+(ROW_LEN*2*EErow), EEvalue+(ROW_LEN*EErow));
		EEPROM_WRITE_STATUS_FLAG = 0;	// next status = idle
	}
}

void _ISR_PSV _SI2CInterrupt(void)	// Slave I2C []
{
	_SI2CIF = 0;	// interrupt flag reset
	
}

/*****************************************************************************/
