/* ////////////////////////////////////////////////////////////////////////////
** Included in "dsODO.c", it contains definitions and variables initialization
/////////////////////////////////////////////////////////////////////////////*/

/*---------------------------------------------------------------------------*/
/* standard	includes													     */
/*---------------------------------------------------------------------------*/
#include "dsODO_common.h"

/*---------------------------------------------------------------------------*/
/* Status bits			    											     */
/*---------------------------------------------------------------------------*/
//Debug on EMUD2/EMUC2
_FICD(0x0003);

/* Run this project using an external crystal routed via the PLL in 
16x multiplier mode 
For the 7.3728 MHz crystal we will derive a throughput of 
7.3728e+6*16/4 = 29.5s MIPS(Fcy) ~33.9 nanoseconds instruction 
cycle time(Tcy).*/
_FOSC(CSW_FSCM_OFF & XT_PLL16);

#ifdef debug_SA	// [15c]
	//Turn off the Watch-Dog Timer.	 [1]		
	_FWDT(WDT_OFF);	
#else
	// WDT Period = 2 ms  Prescale A  Prescale B	-> 2 x 8 x 5 = 80ms
	_FWDT(WDT_ON & WDTPSA_8 & WDTPSB_5);
#endif

//Enable MCLR reset pin and turn on the power-up timers with 64ms delay.
_FBORPOR(MCLR_EN & PWRT_64);				
//Disable Code Protection											
//_FGS(CODE_PROT_OFF);            			 

/*---------------------------------------------------------------------------*/
/* include       														     */
/*---------------------------------------------------------------------------*/
#include "dsODO_prototypes.h"


/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/* Definizioni globali                                                       */
/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

struct Bits{
	unsigned bit0:1;
	unsigned bit1:1;
	unsigned bit2:1;
	unsigned bit3:1;
	unsigned bit4:1;
	unsigned bit5:1;
	unsigned bit6:1;
	unsigned bit7:1;
};

struct Bits VARbits1;
struct Bits VARbits2;

//----- Flags

#define HRT_BT _LATB5
#define LED1 _LATB6
#define LED2 _LATB9
#define CS1	_LATB1		// Generic chip select 1
#define TX_EN1	_LATB2	// TX enable 1
#define CS2	_LATB3		// Generic chip select 2
#define TX_EN2	_LATB4	// TX enable 2
#define MAX_TX_BUFF 32
#define MAX_RX_BUFF 32
#define DISABLE_TX U1STAbits.UTXEN = 0
#define ENABLE_TX U1STAbits.UTXEN = 1
#define DISABLE_TX2 U2STAbits.UTXEN = 0
#define ENABLE_TX2 U2STAbits.UTXEN = 1

const long Tcy = 1000000/(float)(FCY)* 100000000;

//----- Variabili
int i = 0; // generic index
int j = 0; // generic index
long Blink = 0; // heartbeat blink index

// UART1
// Holds the value of uart config reg
unsigned int U1MODEvalue;
// Holds the information regarding uart TX & RX interrupt modes
unsigned int U1STAvalue;
//value to place in the U1BRG register
unsigned int Ubrg;

unsigned char IdRx;					// Motor Controller Id [14]
unsigned char IdTx;
unsigned char IdIndx;				// array index
#define ID_R 1						// right motor Id
#define ID_L 2						// left motor Id
#define ID_I 9						// start-up Id
#define ID_A 0						// Id all (broadcast)
#define R 0							// right index
#define L 1							// right index

unsigned char UartRxBuff[MAX_RX_BUFF];	// serial communication buffer
unsigned char UartTxBuff[MAX_TX_BUFF];
unsigned char UartTmpBuff[MAX_TX_BUFF-4];// temp buffer to compose TX buffer
#define TX_FLAG VARbits1.bit0			// transmission status
#define	TX_BUFF_FULL VARbits1.bit1		// TX bufer status		
unsigned char ChkSum=0;					// checksum
unsigned char UartRxCmd;				// command code
// Rx circular queue pointers
unsigned char UartRxPtrIn=0;	// ingoing bytes captured by ISR
unsigned char UartRxPtrOut=0;	// outgoing bytes read by UartRx function
unsigned char UartRxPtrStart=0;	// message packet starting pointer in queue
unsigned char UartRxPtrEnd=0;	// message packet ending pointer in queue
unsigned char UartRxPtrData=0;	// pointer to first data in queue
unsigned char TmpPtr;			// temporary pointer
unsigned int UartTxCntr=0;
unsigned int UartTxBuffSize=0;
#define UART_RX_TIME_OUT 5				// RX timeout timer in ms
int UartRxTmr = UART_RX_TIME_OUT;
int UartRxStatus =0;					// index for command decoding status
#define OVERRUN_ERROR U1STAbits.OERR	// RX overrun error flag
#define TX_REG_EMPTY  U1STAbits.TRMT	// TX shift register empty
#define FRAME_ERROR U1STAbits.FERR		// RX frame error flag
#define HEADER '@'						// command string header
#define UART_RX_CMD UartRxBuff[2]		// command code
#define UART_RX_CMD_LEN UartRxBuff[3]	// command length
#define RX_ID_FLAG VARbits1.bit3		// command decode flag
#define RX_HEADER_LEN 3					// command string header length (byte) 
#define	TX_PENDING VARbits1.bit2		// TX sequence pending

// UART2
// Holds the value of uart config reg
unsigned int U2MODEvalue;
// Holds the information regarding uart TX & RX interrupt modes
unsigned int U2STAvalue;
//value to place in the U2BRG register
unsigned int Ubrg2;

unsigned char IdRx2;
unsigned char IdIndx2;					// array index
unsigned char UartRx2Buff[MAX_RX_BUFF];	// serial communication buffer
unsigned char UartTx2Buff[MAX_TX_BUFF];
unsigned char UartTmp2Buff[MAX_TX_BUFF-4];// temp buffer to compose TX buffer 
#define TX2_FLAG VARbits1.bit4			// transmission status
#define	TX2_BUFF_FULL VARbits1.bit5		// TX bufer status		
unsigned char ChkSum2=0;				// checksum
unsigned char UartRx2Cmd;				// command code
// Rx circular queue pointers
unsigned char UartRx2PtrIn=0;	// ingoing bytes captured by ISR
unsigned char UartRx2PtrOut=0;	// outgoing bytes read by UartRx function
unsigned char UartRx2PtrStart=0;// message packet starting pointer in queue
unsigned char UartRx2PtrEnd=0;	// message packet ending pointer in queue
unsigned char UartRx2PtrData=0;	// pointer to first data in queue
unsigned char TmpPtr2;			// temporary pointer
unsigned int UartTx2Cntr=0;
unsigned int UartTx2BuffSize=0;
#define UART_RX2_TIME_OUT 5				// RX timeout timer in ms
int UartRx2Tmr = UART_RX2_TIME_OUT;
int UartRx2Status =0;					// index for command decoding status
#define OVERRUN_ERROR2 U2STAbits.OERR	// RX overrun error flag
#define FRAME_ERROR2 U2STAbits.FERR		// RX frame error flag
#define UART_RX2_CMD UartRx2Buff[2]		// command code
#define UART_RX2_CMD_LEN UartRx2Buff[3]	// command length
#define RX2_HEADER_LEN 3				// command string header length (byte) 

int Kpid[3][2];	// speed PID K parameters * 10000

/*
->  [19] and also dsPID folder: descrEng.txt [4] [7]
THIS PARAMETERS ARE VALID FOR RINO ROBOTIC PLATFORM.
HERE JUST AS AN EXAMPLE ON HOW TO CALCULATE THEM
Encoder = 300 cpr 
Gear reduction ratio = 30:1
Wheel speed = 200 rpm
Encoder pulses for each wheel turn = 9.000
Wheel diameter = 58mm -> circumference = 182,2123739mm
Space for each encoder pulse 1x mode Delta S = 0,020245819mm
Space for each encoder pulse 2x mode Delta S = 0,01012291mm
Space for each encoder pulse 4x mode Delta S = 0,005061455mm
Speed calculation K in micron/second = 298536736 -> [19]
*/
#define CPR 300
#define GEAR_RATIO 30
#define CPR_WHEEL CPR*GEAR_RATIO				// 9000
#define DIAMETER  0.058
#define CIRC  DIAMETER*PI						// 0,182212374
#define SPACE_ENC_1X (CIRC / (CPR_WHEEL))
#define SPACE_ENC_2X (CIRC / (CPR_WHEEL * 2))	// 1,012291E-05
#define SPACE_ENC_4X (CIRC / (CPR_WHEEL * 4))
#define K_VEL ((SPACE_ENC_2X) / (TCY))			// TCY = 3,390842E-08

// Speed calculation K in m/s as a power of 2 to semplify dsPID elaboration
long Kvel[2]; // K_VEL << 15
int VelDes[2] = {0,0};	// desired speed mm/s
int VelMes[2] = {0,0};	// measured speed mm/s	
int VelDesM = 0;		// mean measured speed mm/s [23]

// constants for traveled distance calculation: SPACE_ENC_4X in mm
// see [2] in descrEng.exe in dsPID folder
float Ksp[2];
float Axle;// base width, distance between center of the wheels
float SemiAxle; // Axle / 2
int SpTick[2] = {0,0};	// distance traveled by a wheel as encoder pulses
float Spmm[2] = {0,0};	// distance traveled by a wheel as mm(SpTick[x]*Ksp[x])
float Space = 0;		// total distance traveled by the robot
#define SPMIN 0.003		// TO CHECK. Minimum value to perform odometry
float CosPrev =	1;		// previous value for Cos(ThetaMes)
float SinPrev =	0;		// previous value for Sin(ThetaMes)
float PosXmes = 0;		// current X position coordinate
float PosYmes = 0;		// current Y position coordinate
float ThetaMes = 0;		// Current orientation angle
// #define PI 	  3.1415926536
#define TWOPI 6.2831853072

	
// parameters request to Motor Controllers [20]
unsigned char AskParam;
unsigned char AskParamStatus;
#define ASK_PARAM_FLAG VARbits1.bit7
unsigned char AskParamTimer1;	// timer for parameters request
unsigned char AskParamTimer2;	// timer for ALL parameters request
#define	ASK_PARAM_TMO 10		// asks parameters every Xms
#define ASK_PARAM_GUARD (ASK_PARAM_TMO - 1) // guard period before timeout	
#define	ASK_PARAM_ALL_TMO 5		// asks all parameters every N x Xms	

int Curr[2] = {0,0};			// motor current

typedef struct 
{
	unsigned char Low:4;
	unsigned char High:4;
}_Coordinate;

 _Coordinate MapXY [25][50];	// to store field mapping [21]
 int XnormPrev;		// previous X index in 0-49 range
 int YindxPrev;		// previous Y index in 0-49 range
#define SPACE_FLAG VARbits2.bit0
#define ODO_FLAG VARbits2.bit1
#define CELL_SIZE 10			// Dimension of grid cell side in cm


// Angle PID [23]
tPID AnglePIDstruct;
fractional AngleabcCoefficient[3] 
	__attribute__ ((section (".xbss, bss, xmemory")));
fractional AnglecontrolHistory[3] 
	__attribute__ ((section (".ybss, bss, ymemory")));
fractional AngleKCoeffs[] = {0,0,0};
#define ANGLE_KP AngleKCoeffs[0]						// K parameters
#define ANGLE_KI AngleKCoeffs[1]
#define ANGLE_KD AngleKCoeffs[2]
#define ANGLE_PID_DES AnglePIDstruct.controlReference	// desired angle
#define ANGLE_PID_MES AnglePIDstruct.measuredOutput		// measured angle
#define ANGLE_PID_OUT AnglePIDstruct.controlOutput		// PID output
#define MAX_ROT_SPEED 512	// MAX speed (+ or -) of wheels during rotation
#define RAD2DEG 57.295779513	// = 180/PI
#define DEG2RAD 0.0174532925	// = PI/180
float ThetaDes = 0;		// desired orientation angle (set point) [23]
#define ORIENTATION_FLAG VARbits1.bit6	// to control Orientation() execution
char SendVelStatus = 0;					// to control VelDes sending to MCs
// **debug**
// unsigned char ErrNo[2][16];	// store the occurrence of any kind of error 
// unsigned char ErrNo2[16];	// for both MCs and for RX2
// **debug**


// Distance PID [24]
tPID DistPIDstruct;
fractional DistabcCoefficient[3] 
	__attribute__ ((section (".xbss, bss, xmemory")));
fractional DistcontrolHistory[3] 
	__attribute__ ((section (".ybss, bss, ymemory")));
fractional DistKCoeffs[] = {0,0,0};
#define DIST_KP DistKCoeffs[0]						// K parameters
#define DIST_KI DistKCoeffs[1]
#define DIST_KD DistKCoeffs[2]
#define DIST_PID_DES DistPIDstruct.controlReference	// desired distance
#define DIST_PID_MES DistPIDstruct.measuredOutput	// measured distance
#define DIST_PID_OUT DistPIDstruct.controlOutput	// PID output
#define DIST_TIMER_FLAG VARbits2.bit2			// compute distance every X ms
#define DIST_ENABLE_FLAG VARbits2.bit3			// compute distance if enabled
float PosXdes = 0;		// desired X position coordinate
float PosYdes = 0;		// desired Y position coordinate
int Dist;				// distance
#define MIN_DIST_PID 300	// minimum distance to apply Dist PID
int VelDecr = 0;	// speed to subtract to VelDesM to reach the dist target


// Idle time [25]
unsigned long IdleCount = 0;	// how many times it executes main loop
unsigned long IdleSample = 0;	// sampling time to compute idle percentage
#define IDLE_SAMPLE_TIME (ASK_PARAM_TMO * ASK_PARAM_ALL_TMO) // in ms
#define IDLE_TIME_PERIOD 2848	// in ns

// EEPROM
#define EEPROM_SIZE 32	// words reserved in EEPROM space
int _EEDATA(32) EEdata[EEPROM_SIZE];
_prog_addressT EE_addr;

/* an alternative way to read EEprom data
#define eedata_read(base, src, dest) { \
register int eedata_addr;\
register int eedata_val; \
						 \
eedata_addr = __builtin_tbloffset(&base) + \
               ((unsigned)(&src) - (unsigned)(&base)); \
 __asm__("tblrdl [%1], %0" : "=r"(eedata_val) : "r"(eedata_addr)); \
 dest = eedata_val; \
 }
*/

int EEvalue[EEPROM_SIZE];	// EEPROM buffer [27]
char EErow = 0;				// row number to operate on
#define ROW_LEN 16			// length of a EEPROM row
#define EEPROM_WRITE_STATUS_FLAG VARbits2.bit4 // EEPROM write cycle status

unsigned char ResetCount = 0;	// [28]

/* Definitions end ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
