/* ////////////////////////////////////////////////////////////////////////////
** It contains dsPIC settings and initializations
**
** Detailed description are on file "descrEng.txt" 
** numbers between brackets, eg.: [1] , are the references to the specific 
** decription into the file
**
** I commenti dettagliati sono sul file "descrIta.txt" 
** il numero fra parentesi quadre, es.: [1] , e' il riferimento al relativo 
** commento nel documento di descrizione        
/////////////////////////////////////////////////////////////////////////////*/


// standard includes
#include "dsODO_common.h"

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/* ports and peripherals registers setting an initialization                 */
/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
void Settings(void)
{
/*---------------------------------------------------------------------------*/
/* Port	B   			    											     */
/*---------------------------------------------------------------------------*/

// all pins are digital OUT
TRISB  = 0b0000000000000000;
ADPCFG = 0b1111111111111111;

//_TRISB7 = 1;
//_TRISD8 = 1;

/*---------------------------------------------------------------------------*/
/* USART1	[6]     			    									     */
/*---------------------------------------------------------------------------*/
long BaudRate;
//Baud Rate=Fcy/(16*(UxBRG+1)) MAX for dsPIC to dsPIC is 921.600 or 1.843.200
BaudRate = 921600; // baud rate desired 

// Holds the value of uart config reg
extern unsigned int U1MODEvalue;
// Holds the information regarding uart TX & RX interrupt modes
extern unsigned int U1STAvalue;
//value to place in the U1BRG register
extern unsigned int Ubrg;

Ubrg = (FCY/(16*((long)BaudRate)))-1;

// Configure UART1 module to transmit 8 bit data with one stopbit.
// UART_ALTRX_ALTTX definition is wrong in uart.h file,
// substituted with 0xFFE7 value
U1MODEvalue= UART_EN & UART_IDLE_STOP & UART_DIS_WAKE & 0xFFE7 & 
			UART_DIS_ABAUD & UART_NO_PAR_8BIT & UART_2STOPBITS & 
			UART_DIS_LOOPBACK;
U1STAvalue= UART_INT_TX & UART_TX_PIN_NORMAL & UART_TX_ENABLE &
			UART_INT_RX_CHAR & UART_ADR_DETECT_DIS & UART_RX_OVERRUN_CLEAR;
OpenUART1(U1MODEvalue, U1STAvalue, Ubrg);
/*.....................................................................USART1*/

/*---------------------------------------------------------------------------*/
/* USART2	[6d]     			    									     */
/*---------------------------------------------------------------------------*/
long BaudRate2;
// Baud Rate = Fcy / ( 16 * (UxBRG + 1) )
BaudRate2 = 57600; // baud rate desired 

// Holds the value of uart config reg
extern unsigned int U2MODEvalue;
// Holds the information regarding uart TX & RX interrupt modes
extern unsigned int U2STAvalue;
//value to place in the U2BRG register
extern unsigned int Ubrg2;

Ubrg2 = (FCY/(16*((long)BaudRate2)))-1;

// Configure UART2 module to transmit 8 bit data with one stopbit.
U2MODEvalue= UART_EN & UART_IDLE_STOP & UART_DIS_WAKE & 
			UART_DIS_ABAUD & UART_NO_PAR_8BIT & UART_1STOPBIT & 
			UART_DIS_LOOPBACK;
U2STAvalue= UART_INT_TX & UART_TX_PIN_NORMAL & UART_TX_ENABLE &
			UART_INT_RX_CHAR & UART_ADR_DETECT_DIS & UART_RX_OVERRUN_CLEAR;
OpenUART2(U2MODEvalue, U2STAvalue, Ubrg2);
/*.....................................................................USART2*/

/*---------------------------------------------------------------------------*/
/* Timer 1	1ms [13]    			    									 */
/*---------------------------------------------------------------------------*/
OpenTimer1(	T1_ON & 
		T1_GATE_OFF & 
		T1_PS_1_1 & 
		T1_SYNC_EXT_OFF &
		T1_SOURCE_INT, 
		TMR1_VALUE);
/*................................................................ ..Timer 1 */

/*---------------------------------------------------------------------------*/
/* Timer 2	[12]    			    									     */
/*---------------------------------------------------------------------------*/
#define TMR2_VALUE 3
OpenTimer2(	T2_ON & 
			T2_GATE_OFF & 
			T2_PS_1_1 &
			T1_SYNC_EXT_OFF &
			T2_SOURCE_INT, 
			TMR2_VALUE);
/*...................................................................Timer 2 */

/*---------------------------------------------------------------------------*/
/* PWM [5]    		             	    									 */
/*---------------------------------------------------------------------------*/
OpenOC1(OC_IDLE_STOP & OC_TIMER2_SRC & OC_PWM_FAULT_PIN_DISABLE, 2, 2);
/*.......................................................................PWM */

/*---------------------------------------------------------------------------*/
/* I2C [26]    		             	    									 */
/*---------------------------------------------------------------------------*/
unsigned int I2Cconfig2, I2Cconfig1;

/* Baud rate is set for 400 Khz */
//I2Cconfig2 = 46;
/* Configure I2C for 7 bit address mode */
I2Cconfig1 = (I2C_ON & I2C_IDLE_STOP & I2C_CLK_HLD
			& I2C_IPMI_DIS & I2C_7BIT_ADD
			& I2C_SLW_EN & I2C_SM_DIS
			& I2C_GCALL_DIS & I2C_STR_EN);
//OpenI2C(I2Cconfig1,I2Cconfig2);
//I2CADDbits.I2CADD = 0x10;	// I2C slave address

/*.......................................................................I2C */
}

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/* Interrupts setting                                                        */
/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
void ISR_Settings(void)
{
//-------USART1	[6]
ConfigIntUART1(UART_RX_INT_EN & UART_RX_INT_PR4 & UART_TX_INT_EN & 
				UART_TX_INT_PR4);
				
//-------USART2	[6d]
ConfigIntUART2(UART_RX_INT_EN & UART_RX_INT_PR4 & UART_TX_INT_EN & 
				UART_TX_INT_PR4);

//-------I2C	[26]				
//ConfigIntI2C(MI2C_INT_OFF & SI2C_INT_ON & SI2C_INT_PRI_4);

//-------EEPROM	[30]
IEC0bits.NVMIE = 1; // EEPROM Write Complete Interrupt Enable
}

